@@ -2,7 +2,9 @@
 /* Author: David Ayers */
 
 #ifdef __NEXT_RUNTIME__
-/* This test only runs for the GNU runtime. */
+/* This test only runs for the GNU runtime.  TODO: It should work on
+   the NEXT runtime as well (needs testing).
+ */
 
 int main(void)
 {
@@ -11,9 +13,9 @@ int main(void)
 
 #else
 
-#include <objc/objc-api.h>
-#include <objc/Object.h>
-#include <stdio.h>
+#include <objc/runtime.h>
+#include <objc/objc-exception.h>
+#include "../../../objc-obj-c++-shared/TestsuiteObject.m"
 #include <stdlib.h>
 
 static unsigned int handlerExpected = 0;
@@ -31,18 +33,18 @@ my_exception_handler(id excp)
 int 
 main(int argc, char *argv[])
 {
-  _objc_unexpected_exception = my_exception_handler;
+  objc_setUncaughtExceptionHandler (my_exception_handler);
 
   @try
     {
-      @throw [Object new];
+      @throw [TestsuiteObject new];
     }
   @catch (id exc)
     {
       handlerExpected = 1;
     }
 
-  @throw [Object new];
+  @throw [TestsuiteObject new];
   abort();
   return 0;
 }
