@@ -1,6 +1,7 @@
 // Raw memory manipulators -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+// 2009, 2010, 2011
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -49,29 +50,31 @@
  * purpose.  It is provided "as is" without express or implied warranty.
  */
 
-/** @file stl_uninitialized.h
+/** @file bits/stl_uninitialized.h
  *  This is an internal header file, included by other library headers.
- *  You should not attempt to use it directly.
+ *  Do not attempt to use it directly. @headername{memory}
  */
 
 #ifndef _STL_UNINITIALIZED_H
 #define _STL_UNINITIALIZED_H 1
 
-_GLIBCXX_BEGIN_NAMESPACE(std)
+namespace std _GLIBCXX_VISIBILITY(default)
+{
+_GLIBCXX_BEGIN_NAMESPACE_VERSION
 
-  template<bool>
+  template<bool _TrivialValueTypes>
     struct __uninitialized_copy
     {
       template<typename _InputIterator, typename _ForwardIterator>
         static _ForwardIterator
-        uninitialized_copy(_InputIterator __first, _InputIterator __last,
-			   _ForwardIterator __result)
+        __uninit_copy(_InputIterator __first, _InputIterator __last,
+		      _ForwardIterator __result)
         {
 	  _ForwardIterator __cur = __result;
 	  __try
 	    {
 	      for (; __first != __last; ++__first, ++__cur)
-		std::_Construct(&*__cur, *__first);
+		std::_Construct(std::__addressof(*__cur), *__first);
 	      return __cur;
 	    }
 	  __catch(...)
@@ -87,17 +90,17 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
     {
       template<typename _InputIterator, typename _ForwardIterator>
         static _ForwardIterator
-        uninitialized_copy(_InputIterator __first, _InputIterator __last,
-			   _ForwardIterator __result)
+        __uninit_copy(_InputIterator __first, _InputIterator __last,
+		      _ForwardIterator __result)
         { return std::copy(__first, __last, __result); }
     };
 
   /**
    *  @brief Copies the range [first,last) into result.
-   *  @param  first  An input iterator.
-   *  @param  last   An input iterator.
-   *  @param  result An output iterator.
-   *  @return   result + (first - last)
+   *  @param  __first  An input iterator.
+   *  @param  __last   An input iterator.
+   *  @param  __result An output iterator.
+   *  @return   __result + (__first - __last)
    *
    *  Like copy(), but does not require an initialized output range.
   */
@@ -111,25 +114,25 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       typedef typename iterator_traits<_ForwardIterator>::value_type
 	_ValueType2;
 
-      return std::__uninitialized_copy<(__is_pod(_ValueType1)
-					&& __is_pod(_ValueType2))>::
-	uninitialized_copy(__first, __last, __result);
+      return std::__uninitialized_copy<(__is_trivial(_ValueType1)
+					&& __is_trivial(_ValueType2))>::
+	__uninit_copy(__first, __last, __result);
     }
 
 
-  template<bool>
+  template<bool _TrivialValueType>
     struct __uninitialized_fill
     {
       template<typename _ForwardIterator, typename _Tp>
         static void
-        uninitialized_fill(_ForwardIterator __first,
-			   _ForwardIterator __last, const _Tp& __x)
+        __uninit_fill(_ForwardIterator __first, _ForwardIterator __last,
+		      const _Tp& __x)
         {
 	  _ForwardIterator __cur = __first;
 	  __try
 	    {
 	      for (; __cur != __last; ++__cur)
-		std::_Construct(&*__cur, __x);
+		std::_Construct(std::__addressof(*__cur), __x);
 	    }
 	  __catch(...)
 	    {
@@ -144,16 +147,16 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
     {
       template<typename _ForwardIterator, typename _Tp>
         static void
-        uninitialized_fill(_ForwardIterator __first,
-			   _ForwardIterator __last, const _Tp& __x)
+        __uninit_fill(_ForwardIterator __first, _ForwardIterator __last,
+		      const _Tp& __x)
         { std::fill(__first, __last, __x); }
     };
 
   /**
    *  @brief Copies the value x into the range [first,last).
-   *  @param  first  An input iterator.
-   *  @param  last   An input iterator.
-   *  @param  x      The source value.
+   *  @param  __first  An input iterator.
+   *  @param  __last   An input iterator.
+   *  @param  __x      The source value.
    *  @return   Nothing.
    *
    *  Like fill(), but does not require an initialized output range.
@@ -166,86 +169,24 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       typedef typename iterator_traits<_ForwardIterator>::value_type
 	_ValueType;
 
-      std::__uninitialized_fill<__is_pod(_ValueType)>::
-	uninitialized_fill(__first, __last, __x);
+      std::__uninitialized_fill<__is_trivial(_ValueType)>::
+	__uninit_fill(__first, __last, __x);
     }
 
 
-  template<bool>
-    struct __uninitialized_construct_range_dispatch
-    {
-      template<typename _ForwardIterator, typename _Tp>
-        static void
-        __ucr(_ForwardIterator __first, _ForwardIterator __last,
-	      _Tp& __value)
-        {
-	  if(__first == __last)
-	    return;
-
-	  _ForwardIterator __cur = __first;
-	  __try
-	    {
-	      std::_Construct(&*__first, _GLIBCXX_MOVE(__value));
-	      _ForwardIterator __prev = __cur;
-	      ++__cur;
-	      for(; __cur != __last; ++__cur, ++__prev)
-		std::_Construct(&*__cur, _GLIBCXX_MOVE(*__prev));
-	      __value = _GLIBCXX_MOVE(*__prev);
-	    }
-	  __catch(...)
-	    {
-	      std::_Destroy(__first, __cur);
-	      __throw_exception_again;
-	    }
-	}
-    };
-
-  template<>
-    struct __uninitialized_construct_range_dispatch<true>
-    {
-      template<typename _ForwardIterator, typename _Tp>
-        static void
-        __ucr(_ForwardIterator, _ForwardIterator, _Tp&) { }
-    };
-
-  // Constructs objects in the range [first, last).
-  // Note that while these new objects will take valid values,
-  // their exact value is not defined. In particular they may
-  // be 'moved from'.
-  //
-  // While __value may altered during this algorithm, it will have
-  // the same value when the algorithm finishes, unless one of the
-  // constructions throws.
-  //
-  // Requirements: _ForwardIterator::value_type(_Tp&&) is valid.
-  template<typename _ForwardIterator, typename _Tp>
-    inline void
-    __uninitialized_construct_range(_ForwardIterator __first,
-				    _ForwardIterator __last,
-				    _Tp& __value)
-    {
-      typedef typename std::iterator_traits<_ForwardIterator>::value_type
-	_ValueType;
-
-      std::__uninitialized_construct_range_dispatch<
-        __has_trivial_constructor(_ValueType)>::
-	  __ucr(__first, __last, __value);
-    }
-
-
-  template<bool>
+  template<bool _TrivialValueType>
     struct __uninitialized_fill_n
     {
       template<typename _ForwardIterator, typename _Size, typename _Tp>
         static void
-        uninitialized_fill_n(_ForwardIterator __first, _Size __n,
-			     const _Tp& __x)
+        __uninit_fill_n(_ForwardIterator __first, _Size __n,
+			const _Tp& __x)
         {
 	  _ForwardIterator __cur = __first;
 	  __try
 	    {
 	      for (; __n > 0; --__n, ++__cur)
-		std::_Construct(&*__cur, __x);
+		std::_Construct(std::__addressof(*__cur), __x);
 	    }
 	  __catch(...)
 	    {
@@ -260,16 +201,16 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
     {
       template<typename _ForwardIterator, typename _Size, typename _Tp>
         static void
-        uninitialized_fill_n(_ForwardIterator __first, _Size __n,
-			     const _Tp& __x)
+        __uninit_fill_n(_ForwardIterator __first, _Size __n,
+			const _Tp& __x)
         { std::fill_n(__first, __n, __x); }
     };
 
   /**
    *  @brief Copies the value x into the range [first,first+n).
-   *  @param  first  An input iterator.
-   *  @param  n      The number of copies to make.
-   *  @param  x      The source value.
+   *  @param  __first  An input iterator.
+   *  @param  __n      The number of copies to make.
+   *  @param  __x      The source value.
    *  @return   Nothing.
    *
    *  Like fill_n(), but does not require an initialized output range.
@@ -281,8 +222,8 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       typedef typename iterator_traits<_ForwardIterator>::value_type
 	_ValueType;
 
-      std::__uninitialized_fill_n<__is_pod(_ValueType)>::
-	uninitialized_fill_n(__first, __n, __x);
+      std::__uninitialized_fill_n<__is_trivial(_ValueType)>::
+	__uninit_fill_n(__first, __n, __x);
     }
 
   // Extensions: versions of uninitialized_copy, uninitialized_fill,
@@ -300,8 +241,9 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       _ForwardIterator __cur = __result;
       __try
 	{
+	  typedef __gnu_cxx::__alloc_traits<_Allocator> __traits;
 	  for (; __first != __last; ++__first, ++__cur)
-	    __alloc.construct(&*__cur, *__first);
+	    __traits::construct(__alloc, std::__addressof(*__cur), *__first);
 	  return __cur;
 	}
       __catch(...)
@@ -328,6 +270,19 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 					 __result, __alloc);
     }
 
+  template<typename _InputIterator, typename _ForwardIterator,
+	   typename _Allocator>
+    inline _ForwardIterator
+    __uninitialized_move_if_noexcept_a(_InputIterator __first,
+				       _InputIterator __last,
+				       _ForwardIterator __result,
+				       _Allocator& __alloc)
+    {
+      return std::__uninitialized_copy_a
+	(_GLIBCXX_MAKE_MOVE_IF_NOEXCEPT_ITERATOR(__first),
+	 _GLIBCXX_MAKE_MOVE_IF_NOEXCEPT_ITERATOR(__last), __result, __alloc);
+    }
+
   template<typename _ForwardIterator, typename _Tp, typename _Allocator>
     void
     __uninitialized_fill_a(_ForwardIterator __first, _ForwardIterator __last,
@@ -336,8 +291,9 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       _ForwardIterator __cur = __first;
       __try
 	{
+	  typedef __gnu_cxx::__alloc_traits<_Allocator> __traits;
 	  for (; __cur != __last; ++__cur)
-	    __alloc.construct(&*__cur, __x);
+	    __traits::construct(__alloc, std::__addressof(*__cur), __x);
 	}
       __catch(...)
 	{
@@ -361,8 +317,9 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       _ForwardIterator __cur = __first;
       __try
 	{
+	  typedef __gnu_cxx::__alloc_traits<_Allocator> __traits;
 	  for (; __n > 0; --__n, ++__cur)
-	    __alloc.construct(&*__cur, __x);
+	    __traits::construct(__alloc, std::__addressof(*__cur), __x);
 	}
       __catch(...)
 	{
@@ -488,6 +445,168 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
     }
 
 #ifdef __GXX_EXPERIMENTAL_CXX0X__
+  // Extensions: __uninitialized_default, __uninitialized_default_n,
+  // __uninitialized_default_a, __uninitialized_default_n_a.
+
+  template<bool _TrivialValueType>
+    struct __uninitialized_default_1
+    {
+      template<typename _ForwardIterator>
+        static void
+        __uninit_default(_ForwardIterator __first, _ForwardIterator __last)
+        {
+	  _ForwardIterator __cur = __first;
+	  __try
+	    {
+	      for (; __cur != __last; ++__cur)
+		std::_Construct(std::__addressof(*__cur));
+	    }
+	  __catch(...)
+	    {
+	      std::_Destroy(__first, __cur);
+	      __throw_exception_again;
+	    }
+	}
+    };
+
+  template<>
+    struct __uninitialized_default_1<true>
+    {
+      template<typename _ForwardIterator>
+        static void
+        __uninit_default(_ForwardIterator __first, _ForwardIterator __last)
+        {
+	  typedef typename iterator_traits<_ForwardIterator>::value_type
+	    _ValueType;
+
+	  std::fill(__first, __last, _ValueType());
+	}
+    };
+
+  template<bool _TrivialValueType>
+    struct __uninitialized_default_n_1
+    {
+      template<typename _ForwardIterator, typename _Size>
+        static void
+        __uninit_default_n(_ForwardIterator __first, _Size __n)
+        {
+	  _ForwardIterator __cur = __first;
+	  __try
+	    {
+	      for (; __n > 0; --__n, ++__cur)
+		std::_Construct(std::__addressof(*__cur));
+	    }
+	  __catch(...)
+	    {
+	      std::_Destroy(__first, __cur);
+	      __throw_exception_again;
+	    }
+	}
+    };
+
+  template<>
+    struct __uninitialized_default_n_1<true>
+    {
+      template<typename _ForwardIterator, typename _Size>
+        static void
+        __uninit_default_n(_ForwardIterator __first, _Size __n)
+        {
+	  typedef typename iterator_traits<_ForwardIterator>::value_type
+	    _ValueType;
+
+	  std::fill_n(__first, __n, _ValueType());
+	}
+    };
+
+  // __uninitialized_default
+  // Fills [first, last) with std::distance(first, last) default
+  // constructed value_types(s).
+  template<typename _ForwardIterator>
+    inline void
+    __uninitialized_default(_ForwardIterator __first,
+			    _ForwardIterator __last)
+    {
+      typedef typename iterator_traits<_ForwardIterator>::value_type
+	_ValueType;
+
+      std::__uninitialized_default_1<__is_trivial(_ValueType)>::
+	__uninit_default(__first, __last);
+    }
+
+  // __uninitialized_default_n
+  // Fills [first, first + n) with n default constructed value_type(s).
+  template<typename _ForwardIterator, typename _Size>
+    inline void
+    __uninitialized_default_n(_ForwardIterator __first, _Size __n)
+    {
+      typedef typename iterator_traits<_ForwardIterator>::value_type
+	_ValueType;
+
+      std::__uninitialized_default_n_1<__is_trivial(_ValueType)>::
+	__uninit_default_n(__first, __n);
+    }
+
+
+  // __uninitialized_default_a
+  // Fills [first, last) with std::distance(first, last) default
+  // constructed value_types(s), constructed with the allocator alloc.
+  template<typename _ForwardIterator, typename _Allocator>
+    void
+    __uninitialized_default_a(_ForwardIterator __first,
+			      _ForwardIterator __last,
+			      _Allocator& __alloc)
+    {
+      _ForwardIterator __cur = __first;
+      __try
+	{
+	  typedef __gnu_cxx::__alloc_traits<_Allocator> __traits;
+	  for (; __cur != __last; ++__cur)
+	    __traits::construct(__alloc, std::__addressof(*__cur));
+	}
+      __catch(...)
+	{
+	  std::_Destroy(__first, __cur, __alloc);
+	  __throw_exception_again;
+	}
+    }
+
+  template<typename _ForwardIterator, typename _Tp>
+    inline void
+    __uninitialized_default_a(_ForwardIterator __first,
+			      _ForwardIterator __last,
+			      allocator<_Tp>&)
+    { std::__uninitialized_default(__first, __last); }
+
+
+  // __uninitialized_default_n_a
+  // Fills [first, first + n) with n default constructed value_types(s),
+  // constructed with the allocator alloc.
+  template<typename _ForwardIterator, typename _Size, typename _Allocator>
+    void
+    __uninitialized_default_n_a(_ForwardIterator __first, _Size __n, 
+				_Allocator& __alloc)
+    {
+      _ForwardIterator __cur = __first;
+      __try
+	{
+	  typedef __gnu_cxx::__alloc_traits<_Allocator> __traits;
+	  for (; __n > 0; --__n, ++__cur)
+	    __traits::construct(__alloc, std::__addressof(*__cur));
+	}
+      __catch(...)
+	{
+	  std::_Destroy(__first, __cur, __alloc);
+	  __throw_exception_again;
+	}
+    }
+
+  template<typename _ForwardIterator, typename _Size, typename _Tp>
+    inline void
+    __uninitialized_default_n_a(_ForwardIterator __first, _Size __n, 
+				allocator<_Tp>&)
+    { std::__uninitialized_default_n(__first, __n); }
+
+
   template<typename _InputIterator, typename _Size,
 	   typename _ForwardIterator>
     _ForwardIterator
@@ -498,8 +617,7 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       __try
 	{
 	  for (; __n > 0; --__n, ++__first, ++__cur)
-	    ::new(static_cast<void*>(&*__cur)) typename
-		iterator_traits<_ForwardIterator>::value_type(*__first);
+	    std::_Construct(std::__addressof(*__cur), *__first);
 	  return __cur;
 	}
       __catch(...)
@@ -519,10 +637,10 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 
   /**
    *  @brief Copies the range [first,first+n) into result.
-   *  @param  first  An input iterator.
-   *  @param  n      The number of elements to copy.
-   *  @param  result An output iterator.
-   *  @return  result + n
+   *  @param  __first  An input iterator.
+   *  @param  __n      The number of elements to copy.
+   *  @param  __result An output iterator.
+   *  @return  __result + __n
    *
    *  Like copy_n(), but does not require an initialized output range.
   */
@@ -534,6 +652,7 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 					 std::__iterator_category(__first)); }
 #endif
 
-_GLIBCXX_END_NAMESPACE
+_GLIBCXX_END_NAMESPACE_VERSION
+} // namespace
 
 #endif /* _STL_UNINITIALIZED_H */
