@@ -1,4 +1,4 @@
-// Copyright (C) 2001, 2002, 2003, 2009 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, 2009, 2010 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -61,19 +61,19 @@ void test_01()
   // filebuf_type* close()
   std::filebuf* f;
   f = fb_01.close();
-  VERIFY( f != NULL );
+  VERIFY( f );
   VERIFY( !fb_01.is_open() );
 
   f = fb_02.close();
-  VERIFY( f != NULL );
+  VERIFY( f );
   VERIFY( !fb_02.is_open() );
 
   f = fb_03.close();
-  VERIFY( f != NULL );
+  VERIFY( f );
   VERIFY( !fb_03.is_open() );
 
   f = fb_03.close();
-  VERIFY( f == NULL );
+  VERIFY( !f );
   VERIFY( !fb_03.is_open() );
 }
 
